﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WarmlyShip
{
	public class DrawingWarmlyShip
	{
		public EntityWarmlyShip warmlyShip { protected set; get; } //Класс-сущность
		protected float _startPosX; //Координаты отрисовки по оси x
		protected float _startPosY; //Координаты отрисовки по оси y
		private int? _pictureWidth = null; //Ширина окна
		private int? _pictureHeight = null; //Высота окна
		protected readonly int _warmlyShipWidth = 125; //Ширина отрисовки корабля 
		protected readonly int _warmlyShipHeight = 50; //Высота отрисовки корабля

		//Инициализация
		public DrawingWarmlyShip(int speed, float weight, Color bodyColor)
		{
			warmlyShip = new EntityWarmlyShip(speed, weight, bodyColor);
		}

		protected DrawingWarmlyShip(int speed, float weight, Color bodyColor, int warmlyWidth, int warmlyHeight) : this(speed, weight, bodyColor)
		{
			_warmlyShipWidth = warmlyWidth;
			_warmlyShipHeight = warmlyHeight;
		}

		//Начальные коордитанты 
		public void SetPosition(int x, int y, int width, int height)
		{
			if (width < _warmlyShipWidth || height < _warmlyShipHeight) return;
			Random random = new Random();
			_startPosX = x < 0 || x + _warmlyShipWidth > width ? random.Next(0, width - _warmlyShipWidth) : x;
			_startPosY = y < 0 || y + _warmlyShipHeight > height ? random.Next(0, height - _warmlyShipHeight) : y;
			_pictureWidth = width;
			_pictureHeight = height;
		}

		//Движение транспорта по координатам
		public void MoveTransport(Direction direction)
		{
			if (!_pictureWidth.HasValue || !_pictureHeight.HasValue) return;
			switch (direction)
			{
				case Direction.Left: //Влево
					if (_startPosX - warmlyShip.Step > 0) _startPosX -= warmlyShip.Step;
					break;
				case Direction.Up: //Вверх
					if (_startPosY - warmlyShip.Step > 0) _startPosY -= warmlyShip.Step;
					break;
				case Direction.Right: //Вправо
					if (_startPosX + _warmlyShipWidth + warmlyShip.Step < _pictureWidth) _startPosX += warmlyShip.Step;
					break;
				case Direction.Down: //Вниз
					if (_startPosY + _warmlyShipHeight + warmlyShip.Step < _pictureHeight) _startPosY += warmlyShip.Step;
					break;
			}
		}

		//Отрисовка транспорта
		public virtual void DrawTransport(Graphics g)
		{
			if (_startPosX < 0 || _startPosY < 0 || !_pictureHeight.HasValue || !_pictureWidth.HasValue)
			{
				return;
			}
			Pen pen = new(Color.Black, 2);
			SolidBrush brush = new SolidBrush(warmlyShip?.BodyColor ?? Color.White);
			g.FillPolygon(brush, new[]
			{
				new Point((int)(_startPosX), (int)(_startPosY + 20)),
				new Point((int)(_startPosX + _warmlyShipWidth), (int)(_startPosY + 20)),
				new Point((int)(_startPosX + _warmlyShipWidth - 25), (int)(_startPosY + _warmlyShipHeight)),
				new Point((int)(_startPosX + 25), (int)(_startPosY + _warmlyShipHeight))
			});
			g.FillRectangle(brush, _startPosX + _warmlyShipWidth / 5, _startPosY, _warmlyShipWidth * 3 / 5, 20);
			g.FillEllipse(new SolidBrush(Color.Aquamarine), _startPosX + _warmlyShipWidth / 5, _startPosY + 25, 20, 20);
			g.FillEllipse(new SolidBrush(Color.Aquamarine), _startPosX + _warmlyShipWidth * 3 / 5 + 5, _startPosY + 25, 20, 20);
			g.FillEllipse(new SolidBrush(Color.Aquamarine), _startPosX + _warmlyShipWidth * 2 / 5 + 2.5f, _startPosY + 25, 20, 20);
			g.DrawPolygon(pen, new[]
			{
				new Point((int)(_startPosX), (int)(_startPosY + 20)),
				new Point((int)(_startPosX + _warmlyShipWidth), (int)(_startPosY + 20)),
				new Point((int)(_startPosX + _warmlyShipWidth - 25), (int)(_startPosY + _warmlyShipHeight)),
				new Point((int)(_startPosX + 25), (int)(_startPosY + _warmlyShipHeight))
			});
			g.DrawRectangle(pen, _startPosX + _warmlyShipWidth / 5, _startPosY, _warmlyShipWidth * 3 / 5, 20);
			g.DrawEllipse(new(Color.Blue, 2), _startPosX + _warmlyShipWidth / 5, _startPosY + 25, 20, 20);
			g.DrawEllipse(new(Color.Blue, 2), _startPosX + _warmlyShipWidth * 3 / 5 + 5, _startPosY + 25, 20, 20);
			g.DrawEllipse(new(Color.Blue, 2), _startPosX + _warmlyShipWidth * 2 / 5 + 2.5f, _startPosY + 25, 20, 20);
		}

		//Изменение границ отрисовки
		public void ChangeBorders(int width, int height)
		{
			_pictureWidth = width;
			_pictureHeight = height;
			if (_pictureWidth <= _warmlyShipWidth || _pictureHeight <= _warmlyShipHeight)
			{
				_pictureWidth = null;
				_pictureHeight = null;
				return;
			}
			if (_startPosX + _warmlyShipWidth > _pictureWidth)
			{
				_startPosX = _pictureWidth.Value - _warmlyShipWidth;
			}
			if (_startPosY + _warmlyShipHeight > _pictureHeight)
			{
				_startPosY = _pictureHeight.Value - _warmlyShipHeight;
			}

		}

		public (float Left, float Right, float Top, float Bottom) GetCurrentPosition()
		{
			return (_startPosX, _startPosX + _warmlyShipWidth, _startPosY, _startPosY + _warmlyShipHeight);
		}

		public DrawingWarmlyShip ModifShip(int? speed = null, float? weight = null, Color? bodyColor = null)
		{
			return new DrawingWarmlyShip(speed ?? warmlyShip.Speed, weight ?? warmlyShip.Weight, bodyColor ?? warmlyShip.BodyColor);
		}
	}
}
