﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WarmlyShip
{
	internal class MapWithSetShipGeneric<T, U>
		where T : class, IDrawningObject
		where U : AbstractMap
	{
		private readonly int _pictureWidth;
		private readonly int _pictureHeight;
		private readonly int _placeSizeWidth = 210;
		private readonly int _placeSizeHeight = 100;
		private readonly SetShipGeneric<T> _setShips;
		private readonly U _map;

		public MapWithSetShipGeneric(int picWidth, int picHeight, U map)
		{
			int width = picWidth / _placeSizeWidth;
			int height = picHeight / _placeSizeHeight;
			_setShips = new SetShipGeneric<T>(width * height);
			_pictureWidth = picWidth;
			_pictureHeight = picHeight;
			_map = map;
		}

		public static int operator +(MapWithSetShipGeneric<T, U> map, T ship)
		{
			return map._setShips.Insert(ship);
		}

		public static T operator -(MapWithSetShipGeneric<T, U> map, int position)
		{
			return map._setShips.Remove(position);
		}

		public Bitmap ShowSet()
		{
			Bitmap bmp = new(_pictureWidth, _pictureHeight);
			Graphics gr = Graphics.FromImage(bmp);
			DrawShips(gr);
			DrawBackground(gr);
			return bmp;
		}

		public Bitmap ShowOnMap()
		{
			Shaking();
			foreach (var ship in _setShips.GetShips())
			{
				return _map.CreateMap(_pictureWidth, _pictureHeight, ship);
			}
			return new(_pictureWidth, _pictureHeight);
		}

		public Bitmap MoveObject(Direction direction)
		{
			if (_map != null)
			{
				return _map.MoveObject(direction);
			}
			return new(_pictureWidth, _pictureHeight);
		}

		public string GetData(char separatorType, char separatorData)
		{
			string data = $"{_map.GetType().Name}{separatorType}";
			foreach (var ship in _setShips.GetShips())
			{
				data += $"{ship.GetInfo()}{separatorData}";
			}
			return data;
		}

		public void LoadData(string[] records)
		{
			foreach (var rec in records)
			{
				_setShips.Insert(DrawningObjectShip.Create(rec) as T);
			}
		}

		private void Shaking()
		{
			int j = _setShips.Count - 1;
			for (int i = 0; i < _setShips.Count; ++i)
			{
				if (_setShips[i] == null)
				{
					for (; j > i; --j)
					{
						var car = _setShips[j];
						if (car != null)
						{
							_setShips.Insert(car, i);
							_setShips.Remove(j);
							break;
						}
					}
					if (j <= i)
					{
						return;
					}
				}
			}
		}

		private void DrawBackground(Graphics g)
		{
			Pen pen = new(Color.Brown, 8);
			for (int i = 0; i < _pictureWidth / _placeSizeWidth; ++i)
			{
				for (int j = 1; j < _pictureHeight / _placeSizeHeight + 1; ++j) //линия рамзетки места
				{
					g.DrawLine(pen, i * _placeSizeWidth, j * _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth / 2, j * _placeSizeHeight);
					for (int k = 0; k < _placeSizeWidth / (30 * 2); ++k) 
						g.DrawLine(pen, i * _placeSizeWidth + (k + 1) * 30, j * _placeSizeHeight, i * _placeSizeWidth + (k + 1) * 30, j * _placeSizeHeight + 30);
				}
				g.DrawLine(pen, i * _placeSizeWidth, 0, i * _placeSizeWidth, (_pictureHeight / _placeSizeHeight) * _placeSizeHeight);
			}
		}

		private void DrawShips(Graphics g)
		{
			int i = 0;
			foreach (var ship in _setShips.GetShips())
			{
				if (ship != null)
				{
					int temp = 0;
					if (ship.GetCurrentPosition().Bottom - ship.GetCurrentPosition().Top < 75) temp = (int)(ship.GetCurrentPosition().Bottom - ship.GetCurrentPosition().Top);
					ship.SetObject((_pictureWidth / _placeSizeWidth - (i % (_pictureWidth / _placeSizeWidth)) - 1) * _placeSizeWidth, (i / (_pictureWidth / _placeSizeWidth)) * _placeSizeHeight + temp, _pictureWidth, _pictureHeight);
					ship.DrawningObject(g);
				}
				++i;
			}
		}
	}
}
