﻿using System;
using System.Collections.Generic;
using System.DirectoryServices.ActiveDirectory;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WarmlyShip
{
	internal class MapsCollection
	{
		readonly Dictionary<string, MapWithSetShipGeneric<IDrawningObject, AbstractMap>> _mapStorages;

		public List<string> Keys => _mapStorages.Keys.ToList();

		private readonly int _pictureWidth;

		private readonly int _pictureHeight;

		private readonly char separatorDict = '|';

		private readonly char separatorData = ';';

		public MapsCollection(int pictureWidth, int pictureHeight)
		{
			_mapStorages = new Dictionary<string, MapWithSetShipGeneric<IDrawningObject, AbstractMap>>();
			_pictureWidth = pictureWidth;
			_pictureHeight = pictureHeight;
		}

		public void AddMap(string name, AbstractMap map)
		{
			if (_mapStorages.ContainsKey(name)) return;
			_mapStorages.Add(name, new MapWithSetShipGeneric<IDrawningObject, AbstractMap>(_pictureWidth, _pictureHeight, map));
		}

		public void DelMap(string name)
		{
			if (!_mapStorages.ContainsKey(name)) return;
			_mapStorages.Remove(name);
		}

		public MapWithSetShipGeneric<IDrawningObject, AbstractMap> this[string ind]
		{
			get
			{
				if(_mapStorages.ContainsKey(ind)) return _mapStorages[ind];
				return null;
			}
		}

		public void SaveData(string filename)
		{
			if (File.Exists(filename))
			{
				File.Delete(filename);
			}
			using (StreamWriter sw = new(filename))
			{
				sw.Write($"MapsCollection{Environment.NewLine}");
				foreach (var storage in _mapStorages)
				{

					sw.Write($"{storage.Key}{separatorDict}{storage.Value.GetData(separatorDict, separatorData)}{Environment.NewLine}");
				}
			}
		}

		public void LoadData(string filename)
		{
			if (!File.Exists(filename))
			{
				throw new FileNotFoundException("Файл не найден");
			}
			using (StreamReader sr = new(filename))
			{
				string bufferStr;
				bool firstIn = true;
				while ((bufferStr = sr.ReadLine()) != null)
				{
					if (firstIn)
					{
						if (!bufferStr.Contains("MapsCollection"))
						{
							throw new FormatException("Формат данных в файле не правильный");
						}
						else
						{
							_mapStorages.Clear();
						}
						firstIn = false;
					}
					else
					{
						var elem = bufferStr.Split(separatorDict);
						AbstractMap map = null;
						switch (elem[1])
						{
							case "SimpleMap":
								map = new SimpleMap();
								break;
							case "SecondMap":
								map = new SecondMap();
								break;
							case "LastMap":
								map = new LastMap();
								break;
						}
						_mapStorages.Add(elem[0], new MapWithSetShipGeneric<IDrawningObject, AbstractMap>(_pictureWidth, _pictureHeight, map));
						_mapStorages[elem[0]].LoadData(elem[2].Split(separatorData, StringSplitOptions.RemoveEmptyEntries));
					}
				}
			}
		}
	}
}
